@@ -1,8 +1,11 @@
 import csv,re,logging,json
-from rdkit import Chem,DataStructs
-from rdkit.Chem.rdMolDescriptors import GetMorganFingerprint
+from pinky.smiles import smilin
+from pinky.fingerprints import ecfp
+from marshmallow import Serializer, fields
+from cockatoo import metric
 
 logger = logging.getLogger(__name__)
+_mol_cache = {}
 
 class Compound(object):
     """
@@ -19,38 +22,29 @@ class Compound(object):
         :param float conc: Concentration of the compound
         :param str unit: Unit of the concentration (ex. M, % w/v)
         :param float ph: ph of the compound in solution
-        :param float conc_max: Maximum concentration for the compound (default: None)
-        :param str smiles: The SMILES represenation of the compound (default: None)
             
         """
         self.name = name
         self.conc = conc
         self.unit = unit
         self.ph = ph
-        self.conc_min = None
-        self.conc_max = None
         self.molecular_weight = None
         self.density = None
         self.smiles = None
-        self.cations = None
-        self.anions = None
-        self.ions_by_name = {}
-        self.is_peg = False
 
     def mol(self):
-        try:
-            return getattr(self, '_mol')
-        except AttributeError:
-            pass
+        if self.smiles in _mol_cache:
+            return _mol_cache[self.smiles]
 
-        self._mol = None
+        mol = None
         if self.smiles is not None:
             try:
-                self._mol = Chem.MolFromSmiles(self.smiles.encode("utf8"))
+                mol = smilin(self.smiles)
+                _mol_cache[self.smiles] = mol
             except:
                 logger.critical("Invalid smiles format, failed to parse smiles for compound: %s" % self.name)
 
-        return self._mol
+        return mol
 
     def fingerprint(self):
         """
@@ -64,9 +58,9 @@ class Compound(object):
         except AttributeError:
             pass
 
-        self._fp = DataStructs.UIntSparseIntVect(0)
+        self._fp = {}
         if self.mol() is not None:
-            self._fp = GetMorganFingerprint(self.mol(), 2)
+            self._fp = ecfp(self.mol())
 
         return self._fp
 
@@ -87,19 +81,16 @@ class Compound(object):
         # If missing unit bail
         if self.unit is None: return self._molarity
 
-        if re.search(r'w/v', self.unit) and self.molecular_weight is not None:
+        if re.search(r'w/v', self.unit) and self.molecular_weight > 0:
             self._molarity = (self.conc * 10) / float(self.molecular_weight)
-        elif re.search(r'v/v', self.unit) and self.molecular_weight is not None and self.density is not None:
+        elif re.search(r'v/v', self.unit) and self.molecular_weight > 0 and self.density is not None:
             self._molarity =  (self.conc * 0.01) * ((self.density / self.molecular_weight)*1000)
         elif self.unit.lower() == 'm':
             self._molarity = self.conc
 
         return self._molarity
 
-    def reprJSON(self):
-        return self.__dict__
-
-    def __str__(self):
+    def __repr__(self):
         return "[ %s ]" % ", ".join(
             '%r' % i for i in [
                 self.name,
@@ -107,10 +98,6 @@ class Compound(object):
                 self.unit,
                 self.ph,
                 self.smiles,
-                self.is_peg,
-                self.ions_by_name,
-                self.conc_min,
-                self.conc_max,
                 self.molecular_weight,
                 self.density
             ])
@@ -153,19 +140,6 @@ class Cocktail(object):
         """
         self.components.append(compound)
 
-    def map_by_name(self):
-        """
-        Return a map of compound conc and conc_max by name.
-
-        :returns: dict where keys are compound names and values are [conc,conc_max]
-
-        """
-        name_map = {}
-        for c in self.components:
-            name_map[c.name] = c
-
-        return name_map
-
     def fingerprint(self):
         """
         Compute the fingerprint for a cocktail
@@ -183,7 +157,7 @@ class Cocktail(object):
             if cp.fingerprint() is None: continue
             conc_molarity = cp.molarity()
             if conc_molarity == None: conc_molarity = 1
-            for k,v in cp.fingerprint().GetNonzeroElements().iteritems():
+            for k,v in cp.fingerprint().iteritems():
                 self._fp[k] = self._fp.get(k, 0.0) + (float(v) * conc_molarity)
             
         if len(self._fp) == 0:
@@ -191,10 +165,7 @@ class Cocktail(object):
 
         return self._fp
 
-    def reprJSON(self):
-        return self.__dict__
-
-    def __str__(self):
+    def __repr__(self):
         return "[ %s ]" % ", ".join('%r' % i for i in [self.name,len(self),self.ph])
 
 class Screen(object):
@@ -248,70 +219,12 @@ class Screen(object):
         for k in sorted(cmap, key=cmap.get, reverse=True):
             print "%s: %s" % (k, cmap[k])
 
-    def json(self):
-        return json.dumps(self, cls=ComplexEncoder, encoding="utf8")
-
-
-    def _set_ions(self, path):
-        """
-        Set ions from a given ion table.
-
-        The component list of each cocktail is searched for matching ions from
-        the given ion table.
-
-        """
-        ion_table = {}
-        with open(path, 'rb') as csvfile:
-            reader = csv.DictReader(csvfile, delimiter="\t")
-            for row in reader:
-                try:
-                    row['_mol'] = Chem.MolFromSmiles(row['smiles'])
-                except:
-                    logger.critical("Invalid smiles format, failed to parse smiles for ion: %s" % row['name'])
-
-                ion_table[row['name'].lower()] = row
-
-
-        # Match based on substructure search
-        for ck in self.cocktails:
-            for cp in ck.components:
-                cp.cations = []
-                cp.anions = []
-                self._set_mol_ions(cp, ion_table)
-
-                # match ions by name for C6
-                for m in re.split(r'[\s+,\-\/]+', cp.name):
-                    if m in ion_table:
-                        cp.ions_by_name[m] = True
-
-        
-
-    def _set_mol_ions(self, cp, ion_table):
-        """
-        Set ions from a given ion table based on substructure search
-
-        The component list of each cocktail is searched for matching ions from
-        the given ion table using the SMILES for each ion.
-
-        """
-        if cp.smiles is None: return
-        mol = Chem.MolFromSmiles(cp.smiles)
-        if mol is None: return
-
-        for frag in Chem.GetMolFrags(mol, asMols=True):
-            for s in ion_table:
-                if frag.HasSubstructMatch(ion_table[s]['_mol']):
-                    if ion_table[s]['type'] == 'c':
-                        cp.cations.append(Chem.MolToSmiles(frag))
-                    elif ion_table[s]['type'] == 'a':
-                        cp.anions.append(Chem.MolToSmiles(frag))
-
     def _set_summary_stats(self, path):
         """
-        Set summary data for each compound (ex. min,max,std of concentrations).
+        Set summary data for each compound (ex. mw,density,smiles).
 
         """
-        cols = ['conc_min', 'conc_max', 'molecular_weight', 'density']
+        cols = ['molecular_weight', 'density']
         data = {}
         with open(path, 'rb') as csvfile:
             reader = csv.DictReader(csvfile, delimiter="\t")
@@ -320,40 +233,50 @@ class Screen(object):
         
         for ck in self.cocktails:
             for cp in ck.components:
-                if cp.name in data:
-                    row = data[cp.name]
-                    for key in cols:
-                        if key in row and len(row[key]) > 0:
-                            setattr(cp, key, float(row[key]))
-                        else:
-                            setattr(cp, key, None)
-                            logger.info("Missing summary statistic '%s' for compound: %s" % (key, cp.name))
-
-                    if 'smiles' in row and len(row['smiles']) > 0:
-                        cp.smiles = row['smiles'].encode("utf8")
-                        try:
-                            mol = Chem.MolFromSmiles(cp.smiles)
-                        except:
-                            logger.info("Invalid smiles format, failed to parse smiles for compound: %s" % cp.name)
-                    else:
-                        logger.info("Missing smiles data for compound: %s" % cp.name)
-                else:
+                if cp.name not in data:
                     logger.info("Missing summary data for compound: %s" % cp.name)
+                    continue
 
-    def reprJSON(self):
-        return self.__dict__
+                row = data[cp.name]
+                for key in cols:
+                    if key in row and len(row[key]) > 0:
+                        setattr(cp, key, float(row[key]))
+                    else:
+                        setattr(cp, key, None)
 
-    def __str__(self):
+                if 'smiles' in row and len(row['smiles']) > 0:
+                    cp.smiles = row['smiles']
+                    try:
+                        mol = smilin(cp.smiles)
+                    except:
+                        logger.info("Invalid smiles format, failed to parse smiles for compound: %s" % cp.name)
+                else:
+                    logger.info("Missing smiles data for compound: %s" % cp.name)
+
+    def json(self):
+        return ScreenSerializer(self).json
+
+    def __repr__(self):
         return "[ %s ]" % ", ".join([self.name,str(len(self))])
 
 
-# From http://stackoverflow.com/a/5165421
-class ComplexEncoder(json.JSONEncoder):
-    def default(self, obj):
-        if hasattr(obj,'reprJSON'):
-            return obj.reprJSON()
-        else:
-            return json.JSONEncoder.default(self, obj)
+class CompoundSerializer(Serializer):
+    name = fields.String(default=None)
+    conc = fields.Float(default=None)
+    unit = fields.String(default=None)
+    ph = fields.Float(default=None)
+    smiles = fields.String(default=None)
+    molecular_weight = fields.Float(default=None)
+    density = fields.Float(default=None)
+
+class CocktailSerializer(Serializer):
+    components = fields.Nested(CompoundSerializer, many=True)
+    name = fields.String(default=None)
+    ph = fields.Float(default=None)
+
+class ScreenSerializer(Serializer):
+    cocktails = fields.Nested(CocktailSerializer, many=True)
+    name = fields.String(default=None)
 
 def parse_json(path):
     """
@@ -371,7 +294,7 @@ def parse_json(path):
     """
     screen_json = None
     with open(path, 'rb') as f:
-        screen_json = json.load(f, encoding="utf8")
+        screen_json = json.load(f, encoding="utf-8")
 
     if 'name' not in screen_json:
         logger.critical('Invalid json, missing screen name')
@@ -397,7 +320,7 @@ def parse_cocktail(path):
     Parse a cocktail from JSON file
     """
     with open(path, 'rb') as f:
-        ck = json.load(f, encoding="utf8")
+        ck = json.load(f, encoding="utf-8")
         return _parse_cocktail_json(ck)
 
 
@@ -475,7 +398,6 @@ def _parse_cocktail_json(ck):
         for key in compound.__dict__.keys():
             if key.startswith('_'): continue
             if key not in cp:
-                logger.debug('Invalid json, missing compound attribute %s: ' % key)
                 continue
             setattr(compound, key, cp[key])
 
@@ -541,8 +463,6 @@ def _parse_cocktail_csv(row):
             ph_vals.append(compound.ph)
 
         matches = re.search(r'^((?:peg|polyethylene\sglycol)[^\d]+)(\d+)', compound.name)
-        if matches:
-            compound.is_peg = True
 
         # handle special case for tacsimate
         if re.search(r'tacsimate', compound.name):
@@ -594,3 +514,57 @@ def _create_tacsimate(cp):
         ))
 
     return mixture
+
+def distance(screen1, screen2, weights):
+    """
+    Compute the distance between two screens (from Newman et al. 2010).
+
+    :param screen screen1: First screen
+    :param screen screen2: Second screen
+    :param array weights: weights
+
+    :returns: The distance score between 0 and 1
+        
+    """
+    sum1 = 0.0
+    for c1 in screen1.cocktails:
+        mini = 100000000
+        for c2 in screen2.cocktails:
+            distance = metric.distance(c1, c2, weights)
+            if mini > distance:
+                mini = distance
+        sum1 += mini
+
+    sum2 = 0.0
+    for c1 in screen2.cocktails:
+        mini = 100000000
+        for c2 in screen1.cocktails:
+            distance = metric.distance(c1, c2, weights)
+            if mini > distance:
+                mini = distance
+        sum2 += mini
+
+    score = ( (sum1/float(len(screen1))) + (sum2/float(len(screen2))) )/2.0
+    return score
+
+def internal_similarity(s, weights):
+    """
+    Compute the internal diversity within a screen (from Newman et al. 2010).
+
+    :param screen s: The screen
+    :param array weights: weights
+
+    :returns: The diversity score between 0 and 1
+        
+    """
+    sum_avg = 0
+    for c1 in s.cocktails:
+        isum = 0
+        n = 0
+        for c2 in s.cocktails:
+            isum += metric.distance(c1, c2, weights)
+            n += 1
+
+        sum_avg += isum / n
+
+    return sum_avg / len(s)
